local L = AceLibrary("AceLocale-2.2"):new("Automaton_Ding")
Automaton_Ding = Automaton:NewModule("Ding", "AceEvent-2.0", "AceConsole-2.0", "AceDebug-2.0")

L:RegisterTranslations("enUS", function() return {
    ["Ding"] = true,
    ["Notification on level up"] = true,
    ["Enable"] = true,
    ["Enable module"] = true,
    ["Message"] = true,
    ["Notification message"] = true,
    ["Notification usage"] = [[

$L - New Level
$l - Old Level
$C - Class
$R - Race
$N - Name]],
    ["Woot, ding $L!"] = true, -- default message
    ["Channels"] = true,
    ["Channels to notify"] = true,
    ["Notify %s channel"] = true,
    ["Multiple"] = true,
    ["Notify only when player level is a positive multiple of this number"] = true,
} end)

local types = {"say", "emote", "yell", "party", "guild", "officer", "raid", "battleground"}
local gsubtbl = {}

function Automaton_Ding:OnInitialize()
    local opts = {
        type = "group",
        name = L["Ding"],
        desc = L["Notification on level up"],
        args = {
            enable = {
                type  = "toggle",
                name  = L["Enable"],
                desc  = L["Enable module"],
                get   = function() return Automaton:IsModuleActive("Ding") end,
                set   = function(v) Automaton:ToggleModuleActive("Ding", v) end,
                order = 10,
            },
            message = {
                type  = "text",
                name  = L["Message"],
                desc  = L["Notification message"],
                usage = L["Notification usage"],
                get   = function() return self.db.profile.message end,
                set   = function(v) self.db.profile.message = v end,
                order = 20,
            },
            multiple = {
                type  = "range",
                name  = L["Multiple"],
                desc  = L["Notify only when player level is a positive multiple of this number"],
                min   = 1,
                max   = MAX_PLAYER_LEVEL,
                step  = 1,
                get   = function() return self.db.profile.multiple end,
                set   = function(v) self.db.profile.multiple = v end,
                order = 30,
            },
            channels = {
                type  = "group",
                name  = L["Channels"],
                desc  = L["Channels to notify"],
                args  = {},
                order = 40,
            },
        },
    }

    for _,t in ipairs(types) do
        -- *Cough* Please fix pass = true with args being toggles, ckk :)
        opts.args.channels.args[t] = {
            type = "toggle",
            name = t,
            desc = L["Notify %s channel"]:format(t),
            get  = function()
                return self.db.profile.channels[t]
            end,
            set  = function(v)
                self.db.profile.channels[t] = v
            end,
        }
    end

    Automaton.options.args.ding = opts

    self.db = Automaton:AcquireDBNamespace("Ding")
    Automaton:RegisterDefaults("Ding", "profile", {
        message  = L["Woot, ding $L!"],
        multiple = 1,
        channels = {
            guild = true,
            party = true,
        },
    })
end

function Automaton_Ding:OnEnable()
    if not self.db.profile.loaded then
        -- Default to disabled
        Automaton:ToggleModuleActive("Ding", false)
        self.db.profile.loaded = true
        return
    end

    self:RegisterEvent("PLAYER_LEVEL_UP")

    gsubtbl["$R"] = UnitRace("player")
    gsubtbl["$N"] = UnitName("player")
end

function Automaton_Ding:PLAYER_LEVEL_UP(level, hp, mp, tp, str, agi, sta, int, spi)
    if level % self.db.profile.multiple ~= 0 then
        self:Debug(("Skipping notification: %s is not a positive multiple of %s"):format(level, self.db.profile.multiple))
        return
    end

    gsubtbl["$L"] = tostring(level)
    gsubtbl["$l"] = tostring(level - 1)
    local msg = self.db.profile.message:gsub("($%w)", gsubtbl)
    for k,v in ipairs(types) do
        if self.db.profile.channels[v] then
            self:Debug(("Sending `%s' to channel `%s'."):format(msg, v:upper()))
            SendChatMessage(msg, v:upper())
        end
    end
end
